#include "ppc.h"
#include "m33.h"

PPC::PPC(float hfov, int _w, int _h) {

  w = _w;
  h = _h;
  a = V3(1.0f, 0.0f, 0.0f);
  b = V3(0.0f, -1.0f, 0.0f);
  float hfovRadians = hfov / 180.0f * 3.1415f;
  c = V3((float)-w/2.0f, (float)h/2.0f, -(float)w/(2.0f*tan(hfovRadians/2.0f)));
  C = V3(0.0f, 0.0f, 0.0f);

}

bool PPC::Project(V3 P, V3& projP) {

  M33 cam;
  cam.SetColumn(0, a);
  cam.SetColumn(1, b);
  cam.SetColumn(2, c);
  V3 q = cam.Inverted() * (P-C);
  if (q[2] < 0.0f)
    return false;

  projP[0] = q[0] / q[2];
  projP[1] = q[1] / q[2];
  projP[2] = 1.0f / q[2];
  return true;

}

void PPC::Draw(float actualFocalLength, FrameBuffer *fb, PPC *ppc) {

  V3 P[4];
  P[0] = Unproject(0.0f, 0.0f, actualFocalLength);
  P[1] = Unproject((float)w, 0.0f, actualFocalLength);
  P[2] = Unproject((float)w, (float)h, actualFocalLength);
  P[3] = Unproject(0.0f, (float)h, actualFocalLength);

  V3 black(0.0f, 0.0f, 0.0f);
  V3 red(1.0f, 0.0f, 0.0f);

  for (int i = 0; i < 4; i++) {
    fb->Draw3DSegment(P[i], P[(i+1)%4], black, black, ppc);
    fb->Draw3DSegment(P[i], C, black, red, ppc);
  }

}

V3 PPC::GetVD() {

  V3 ret = (a^b).UnitVector();
  return ret;

}

float PPC::Getf() {

  return c*GetVD();

}

V3 PPC::Unproject(float uf, float vf, float distance) {

  float f = Getf();
  V3 ret = C + (c + a*uf + b*vf) * distance / f;
  return ret;

}

